"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
const dependency_container_1 = require("../dependency-container");
const injectionTokenMetadataKey = "injectionTokens";
function injectable() {
    return function (target) {
        const params = Reflect.getMetadata("design:paramtypes", target) || [];
        const injectionTokens = Reflect.getOwnMetadata(injectionTokenMetadataKey, target) || {};
        Object.keys(injectionTokens).forEach(key => {
            params[+key] = injectionTokens[key];
        });
        dependency_container_1.typeInfo.set(target, params);
    };
}
exports.injectable = injectable;
function autoInject() {
    return function (target) {
        injectable()(target);
        return class extends target {
            constructor(...args) {
                super(...args.concat(dependency_container_1.typeInfo.get(target).slice(args.length).map(type => dependency_container_1.instance.resolve(type))));
            }
        };
    };
}
exports.autoInject = autoInject;
function inject(token) {
    return function (target, _propertyKey, parameterIndex) {
        const injectionTokens = Reflect.getOwnMetadata(injectionTokenMetadataKey, target) || {};
        injectionTokens[parameterIndex] = token;
        Reflect.defineMetadata(injectionTokenMetadataKey, injectionTokens, target);
    };
}
exports.inject = inject;
function registry(providers = [], container) {
    return function (target) {
        providers.forEach(provider => (container || dependency_container_1.instance).register(provider));
        return target;
    };
}
exports.registry = registry;
